#!/bin/sh
# Common Ml Digest Transmitter (CMDT)
#       Copyright 1993-2004 (C) SAITO Yasuhiro.
#       All rights reserved.
#

VERSION=@VER@

#
# Set path of sendmail
#
SENDMAIL=@SENDMAIL@

#
# Set local environment.
#
DRIVER=@DRIVER@
DOMAIN=@DOMAIN@
MSPOOL=@MSPOOL@
TMPDIR=@TMPDIR@
PATH=@PATH@
GREP=@GREP@
export PATH
umask 022

unset LANG >/dev/null 2>&1

# Get name of Mailing List.
ML=$1

PIDF=$DRIVER/$ML/cmdt.pid
BATT=$DRIVER/$ML/batchtime
DLST=$DRIVER/$ML/$ML-digest
HEAD=$TMPDIR/DGST.$$

# Member list file
LIST=$DRIVER/$ML/$ML-member

# Mail deliver list
DELV=$DRIVER/$ML/$ML-list

if [ ! -f $LIST ]
then
        cat $DELV > $LIST
	egrep "^[0-9]* SEND" $LIST | sed -e 's/.*<//' -e 's/>.*//' > $DELV
fi

DGSTFILE=$MSPOOL/$ML/spool/.digest
DGSTDIST=$MSPOOL/$ML/spool/.digest2
DGSTLOCK=$MSPOOL/$ML/spool/.digest.lock

trap "rm -f $PIDF $HEAD $DLST $DGSTLOCK ; exit 0" 0 1 2 3 15
if [ -f $PIDF ]
then
	exit 0
fi

# Lock key.
LKEY=$MSPOOL/$ML/spool/.DLOCK

# Make PID File
echo $$ > $PIDF

# Set default value.
. $DRIVER/config.default

if [ -f $DRIVER/$ML/config.$ML ]
then
	. $DRIVER/$ML/config.$ML
fi

# Check change domain name.
if [ "$GENERICFROM" ]
then
	DOMAIN=$GENERICFROM
fi

# Make unix-from address.
if [ ! "$UNIXFROM" ]
then
    UNIXFROM=$ML-request@$DOMAIN
fi

if [ ! -f $BATT ]
then
	echo "03:00" > $BATT
fi

BSIZE=0
if [ "$DIGESTMAX" ]
then
	if [ $DIGESTMAX -gt 0 ]
	then
		BSIZE=`expr $DIGESTMAX \* 1024`
	fi
fi

# Check change Reply-To:
if [ "$REPLYTO" ]
then
	REPLYTO=$REPLYTO
else
	REPLYTO=$ML@$DOMAIN
fi

while :
do

!!	TIME=`date | tr ' ' '\012' | egrep '[0-9][0-9]:[0-9][0-9]:[0-9][0-9]'`
!!	TIME=`echo "$TIME." | sed -e 's/:[0-9][0-9]\.//'`
!!
@@	TIME=`date '+%H:%M'`
@@
--	TIME=`telnet 127.0.0.1 13 2>/dev/null | \
--		tr ' ' '\012' | egrep '[0-9][0-9]:[0-9][0-9]:[0-9][0-9]'`
--	TIME=`echo "$TIME." | sed -e 's/:[0-9][0-9]\.//'`
--
	BATCH=0
	$GREP $TIME $BATT 2>&1 >/dev/null
	if [ $? -eq 0 ]
	then
		BATCH=1
	fi

	if [ $BSIZE -ne 0 ]
	then
		set `wc -c $DGSTFILE` 0
		if [ $1 -ge $BSIZE ]
		then
			 BATCH=1
		fi
	fi

	if [ "$DIGESTLEN" ]
	then
		set `wc -l $DGSTFILE` 0
		if [  $1 -ge $DIGESTLEN ]
		then
			BATCH=1
		fi
	fi

	if [ $BATCH -eq 1 ]
	then
		# Check member of digest mode.
		$GREP DGST $LIST >/dev/null 2>&1
		if [ $? -ne 0 ]
		then
			rm -f $DGSTFILE
			exit 0
		fi

		egrep "^[0-9]* DGST" $LIST | \
		sed -e 's/.*(//' -e 's/).*//' > $DLST

		if [ -f $DGSTFILE ]
		then
			# Get lock.
			DELAY=`expr $$ % 6`

			# Get delay time.
			case "$DELAY" in
			0)	set 1 2 3 3 2 1 ;;
			1)	set 1 3 2 1 2 3 ;;
			2)	set 2 1 3 1 3 2 ;;
			3)	set 2 3 1 2 1 3 ;;
			4)	set 3 1 2 2 3 1 ;;
			*)	set 3 2 1 3 1 2 ;;
			esac

			umask 077
			until ln $LKEY $DGSTLOCK >/dev/null 2>&1
			do
				DELAY=$1
				shift
				set $@ $DELAY
				echo "sleep $DELAY"
				sleep $DELAY
			done
			# Lock comprele.
			umask 022

			cat $DGSTFILE > $DGSTDIST
			rm -f $DGSTFILE

			echo ""                                       >> $DGSTDIST
			echo "-------- End Digest of $ML ML --------" >> $DGSTDIST

			echo "From: $ML-request@$DOMAIN ($ML ML admin)" > $HEAD
			echo "To: $ML@$DOMAIN ($ML ML Digest mode)"    >> $HEAD
			echo "Subject: Digest of $ML ML."              >> $HEAD
			case $REPLYSW in
			1|2)
				echo "Reply-To: $REPLYTO"              >> $HEAD
				;;
			esac
			echo "Errors-To: $ML-request@$DOMAIN"          >> $HEAD
			echo "X-ML-Driver: CMDT (Version $VERSION)"    >> $HEAD

			if [ "$SENDER" ]
			then
				echo "Sender: $SENDER"                 >> $HEAD
			fi

			if [ "$PRECEDENCE" ]
			then
				echo "Precedence: $PRECEDENCE"         >> $HEAD
			fi

			echo ""                                        >> $HEAD

			cat $HEAD $DGSTDIST | \
			$SENDMAIL -oi -f$UNIXFROM -oem -odb $ML-digest-dist

			sleep 60
			
			rm -f $DGSTDIST
			rm -f $DGSTLOCK

			exit 0
		fi
	fi

	sleep 55
done

exit 0
